--
-- $RCSfile: memory.vhd,v $
-- $Revision: 1.4 $
-- $Author: petera $
-- $Date: 90/06/21 10:32:11 $
--

use work.dp32_types.all;

entity memory is
  generic (Tpd : Time := unit_delay);
  port (d_bus : inout bus_bit_32 bus;
	a_bus : in bit_32;
	read, write : in bit;
	ready : out bit);
end memory;

architecture behaviour of memory is
  
begin

  process 
  
    constant low_address : integer := 0;
    constant high_address : integer := 65535;
    type memory_array is
      array (integer range low_address to high_address) of bit_32;
      
    variable mem : memory_array;
    variable address : integer;
    
  begin
    --
    -- put d_bus and reply into initial state
    --
    d_bus <= null after Tpd;
    ready <= '0' after Tpd;
    --
    -- wait for a command
    --
    wait until (read = '1') or (write = '1');
    --
    -- dispatch read or write cycle
    --
    address := bits_to_int(a_bus);
    if address >= low_address and address <= high_address then
      -- address match for this memory
      if write = '1' then
	ready <= '1' after Tpd;
      	-- wait until end of write cycle
	wait until write = '0';
	-- sample data from Tpd ago
	mem(address) := d_bus'delayed(Tpd);
      else -- read = '1'
	-- fetch data
	d_bus <= mem(address) after Tpd;
	ready <= '1' after Tpd;
	-- hold for read cycle
	wait until read = '0';
      end if;
    end if;
  end process;

end behaviour;
